// Explorer Bar Utilities Version 1.5
// Created by CadWizard
// Copyright (c) Larry Williams II. All rights reserved.
// Email @ cadwizardx@hotmail.com
// filename:ExplorerBarUtilities.js;datetime:9:14 PM Saturday, July 10, 2010
// resources:ExplorerBarUtilities.jsExpBarIcons.icl;InstallCopyMoveTo.js;InstallShowSwitch.js;InstallUnblockFiles.js
//
// Provides functions for buttons placed on the Windows Vista/7 Explorer Bar (alongside
// the 'Share' and 'Burn' buttons, not IE Explorer).
//
// The included functions handle five (5) different button actions with the following intent......

// ranslated to Russian by Vadim Sterkin
// (Copy to folder)	: Copies selected items in an Explorer Window to an user selected location.
// (Move to folder)	: Moves selected items in an Explorer Window to an user selected location.

//   , ..     http://www.outsidethebox.ms/10656/
//   http://www.sevenforums.com/customization/96851-add-custom-explorer-bar-buttons-actions.html
// (Show desktop)	: Provides the same function as the 'Quick Launch Show desktop' shortcut on the taskbar.
// (Switch windows)	: Provides the same function as the 'Quick Launch Switch windows' shortcut on the taskbar (3DFlip).
// (Unblock files)	: Traverses the folder of an Explorer Window's files and folders and unblocks any files that me be blocked.
//
// This application provided free of charge for personal use without implied or expressed warranty.
// This unmodified application may be freely distributed without charge or reimbursement via the Internet only and
// may not be included in any other form of digital media including USB Memory Stick's, CD's and or DVD's. 
// This unmodified application may not be bundled or packaged within any type of application suite, library or archive
// without first obtaining prior written consent from the author.
// This code may be modified to be included in your own works free of charge without written consent
// as long as the original author is given credit for his effort by including the top commented section along
// with a brief explanation of the modifications and how any modifications have affected the intent, execution and
// end result of the original application.
//
// Author not responsible for any harm or damage that may occur from the use of this application whether
// accidental or intentional.
// The code contained herin has access to the filesystem and network resources, caution should be taken
// whenever executing this application.
// Be sure, be safe, If you have a doubt, DON'T DO IT!
// Create a Restore Point, backup registry and important files and data prior to installing this application.
// Enjoy!!
//

// attach to the Win32Shell
var wshShell = new ActiveXObject("WScript.Shell");
// get command line parameters
var wshArgs = WScript.Arguments;

// program constants
var appTITLE = "Explorer Bar Utilities";
var ssfDESKTOP = 0;
var ssfPERSONAL = 5;	//Documents
var ssfDRIVES = 17;
var ssfWINDOWS = 36;
var ssfPROFILE = 40;

// make sure there's something to do
if (wshArgs.Count() >= 1)
{
	DelegateTaskAction();
}

// detach from the Win32Shell
// last line of execution
wshShell = null;

// determine action and call correct handler
function DelegateTaskAction()
{
	switch (wshArgs(0))
	{
		case "CopyTo":
		case "MoveTo":
			CopyMoveToHandler();
			break;
		case "ToggleDesktop":
			ToggleDesktopHandler();
			break;
		case "WindowSwitcher":
			WindowSwitcherHandler();
			break;
		case "UnblockFiles":
			UnblockFiles();
			break;
		default:
			break;
	}
}

// object for managing complex browse dialog
function DialogCaption()
{
	this.mainCaption = " ,     ";
	this.endCaption = "\r\n   OK.";
	this.actionCaption = "copy ";
	this.itemCaption = "  ";
	
	this.Caption = function()
	{
		return this.mainCaption + this.actionCaption + this.itemCaption + this.endCaption;
	}
}

// function to copy/move the selected items to an user specified location
function CopyMoveToHandler()
{
	var dc = new DialogCaption();
	var fso = new ActiveXObject("Scripting.FileSystemObject");
	var destPath = "";
	var bCopy = false;
	
	// set the browse dialog based on action and the number of items selected
	if (wshArgs.Count() > 3)
	{
		dc.itemCaption = "these " + (wshArgs.Count() - 2) + " items.";
	}
	else
	{
		dc.itemCaption = wshArgs(2) + ".";
	}
	switch (wshArgs(0))
	{
		case "CopyTo":
		//translated copy
			dc.actionCaption = " ";
			bCopy = true;
			break;
		case "MoveTo":
			dc.actionCaption = " ";
			break;
		default:
			break;
	}
	
	// browse to select the destination folder for the selected items
	var result = ShellBrowseForFolderJS(0, dc.Caption(), ssfDESKTOP);
	if (result != null)
	{
		var bYes = false;
		destPath = result.self.path;
		for (var i = 2; i < wshArgs.Count(); i++)
		{
			var str = wshArgs(i);
			var shellFolder = null;
			var shellFile = null;
			
			try
			{
				// it's a file
				shellFile = fso.GetFile(str);
			}
			catch (e)
			{
				// it's an error, so it must be a folder
				shellFolder = fso.GetFolder(str);
			}
			var shellObject = null;
			if (shellFile != null)
			{
				// it's a file
				shellObject = shellFile;
			}
			else if (shellFolder != null)
			{
				// it's a folder
				shellObject = shellFolder;
			}
			if (bCopy)
			{
				try
				{
					// it's copied
					shellObject.Copy(destPath + "\\" + shellObject.Name, false);
				}
				catch (e)
				{
					// it's an error, so it must already exist
					if (bYes != true)
					{
						// replace?
						var ret = wshShell.Popup(" " + shellObject.Name + "  .\r\n   ?", 30, wshArgs(0), 3 + 32);
						if (ret == 2)
						{
							// canceled by the user
							break;
						}
						else if (ret == 6)
						{
							shellObject.Copy(destPath + "\\" + shellObject.Name, true);
							if (wshArgs.Count() - 2 > 1)
							{
								// if so,
								ret = wshShell.Popup("         ?", 30, wshArgs(0), 4 + 32);
								bYes = (ret == 6);
							}
						}
					}
					else
					{
						// do the same for any further conflicts
						shellObject.Copy(destPath + "\\" + shellObject.Name, true);
					}
				}
			}
			else
			{
				try
				{
					// it's moved
					shellObject.Move(destPath + "\\" + shellObject.Name);
				}
				catch (e)
				{
					// it's an error, so it must already exist
					if (bYes != true)
					{
						// replace?
						var ret = wshShell.Popup(" " + shellObject.Name + "  .\r\n   ?", 30, wshArgs(0), 3 + 32);
						if (ret == 2)
						{
							// canceled by the user
							break;
						}
						else if (ret == 6)
						{
							shellObject.Copy(destPath + "\\" + shellObject.Name, true);
							shellObject.Delete(true);
							if (wshArgs.Count() - 2 > 1)
							{
								// if so,
								ret = wshShell.Popup("         ?", 30, wshArgs(0), 4 + 32);
								bYes = (ret == 6);
							}
						}
					}
					else
					{
						// do the same for any further conflicts
						shellObject.Copy(destPath + "\\" + shellObject.Name, true);
						shellObject.Delete(true);
					}
				}
			}
		}
	}
	fso = null;
	// done
}

// function for managing the BrowseForFolder dialog
function ShellBrowseForFolderJS(hWnd,caption,root)
{
	var ret = null;
	var appShell = new ActiveXObject("shell.application");
	var shellFolder = appShell.BrowseForFolder(hWnd, caption, 0, root);
	if (shellFolder != null)
	{
		ret = shellFolder;
	}
	appShell = null;
	shellFolder = null;
	return ret;
}

// function for toggling the desktop visibility
function ToggleDesktopHandler()
{
	var appShell = new ActiveXObject("shell.application");
	appShell.ToggleDesktop();
	appShell = null;
}

// function for activating the window switcher application
function WindowSwitcherHandler()
{
	var appShell = new ActiveXObject("shell.application");
	appShell.WindowSwitcher();
	appShell = null;
}

// function for unblocking downloaded files
function UnblockFiles()
{
	var fso = new ActiveXObject("Scripting.FileSystemObject");
	var currentFolder = fso.GetFolder(wshArgs(1));
	// start with the first dominoe, i mean folder
	RecurseFolder(currentFolder);
	
	// recursive function to traverse the current folder dir tree and files
	function RecurseFolder(fldr)
	{
		//WScript.echo(fldr.Path);
		
		var currentFiles = new Enumerator(fldr.files);
		if (currentFiles != null)
		{
			for (; !currentFiles.atEnd(); currentFiles.moveNext())
			{
				// check to see if the zone identifier exists 
				var fileName = currentFiles.item().Path + ":Zone.Identifier";
				try
				{
					var file1 = fso.OpenTextFile(fileName, 2);
					file1.Close();
				}
				catch(e){ }  //  it's an error, so it must not
			}
		}
		// get subfolders and call recusive function to 
		// traverse each ones folders and files
		var currentFolders = new Enumerator(fldr.subfolders);
		if (currentFolders != null)
		{
			for (; !currentFolders.atEnd(); currentFolders.moveNext())
			{
				RecurseFolder(currentFolders.item());
			}
		}
	}
	
	currentFolder = null;
	fso = null;
	wshShell.Popup("Finished unblocking all files", 30, wshArgs(0), 64);
}